package com.muyu.data.source.client.pool.config;


import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.muyu.common.core.annotation.Excel;
import com.muyu.data.source.client.pool.BaseConfig;
import com.muyu.data.source.domain.SysDataSource;
import io.swagger.annotations.ApiModelProperty;
import lombok.Builder;
import lombok.Data;

/**
 * mysql连接池配置
 *
 * @author WangLei
 * @Date 2024/4/29 029 14:24
 */
@Data
@Builder
public class MysqlPoolConfig extends BaseConfig {

    private static final long serialVersionUID = 1L;

    /** 数据管理id */
    @TableId(value = "id",type = IdType.AUTO)
    @ApiModelProperty(name = "数据管理id", value = "数据管理id")
    private Long id;

    /** 接入源名称 */
    @Excel(name = "接入源名称")
    @ApiModelProperty(name = "接入源名称", value = "接入源名称")
    private String accessSourceName;

    /** 数据来源系统名称 */
    @Excel(name = "数据来源系统名称")
    @ApiModelProperty(name = "数据来源系统名称", value = "数据来源系统名称")
    private String dataSourceSystemName;

    /** 主机地址 */
    @Excel(name = "主机地址")
    @ApiModelProperty(name = "主机地址", value = "主机地址")
    private String hostAddress;

    /** 主机端口 */
    @Excel(name = "主机端口")
    @ApiModelProperty(name = "主机端口", value = "主机端口")
    private String hostPort;

    /** 数据连接类型ID */
    @Excel(name = "数据连接类型ID")
    @ApiModelProperty(name = "数据连接类型ID", value = "数据连接类型ID")
    private Long dataAccessTypeId;

    /** 数据库名称 */
    @Excel(name = "数据库名称")
    @ApiModelProperty(name = "数据库名称", value = "数据库名称")
    private String databaseName;

    /** 数据连接参数 */
    @Excel(name = "数据连接参数")
    @ApiModelProperty(name = "数据连接参数", value = "数据连接参数")
    private String dataConnectionParameter;

    /** 初始连接数量 */
    @Excel(name = "初始连接数量")
    @ApiModelProperty(name = "初始连接数量", value = "初始连接数量")
    private String initialQuantity;

    /** 最大连接数量 */
    @Excel(name = "最大连接数量")
    @ApiModelProperty(name = "最大连接数量", value = "最大连接数量")
    private String maximumQuantity;

    /** 最大等待时间 */
    @Excel(name = "最大等待时间")
    @ApiModelProperty(name = "最大等待时间", value = "最大等待时间")
    private Long maximumTime;

    /** 最大等待次数 */
    @Excel(name = "最大等待次数")
    @ApiModelProperty(name = "最大等待次数", value = "最大等待次数")
    private String maximumFrequency;

    /** 数据库用户名 */
    @Excel(name = "数据库用户名")
    @ApiModelProperty(name = "数据库用户名", value = "数据库用户名")
    private String databaseUserName;

    /** 数据库用户密码 */
    @Excel(name = "数据库用户密码")
    @ApiModelProperty(name = "数据库用户密码", value = "数据库用户密码")
    private String databaseUserPassword;

    /** 驱动名称 */
    @Excel(name = "驱动名称")
    @ApiModelProperty(name = "驱动名称", value = "驱动名称")
    private String driverName;

    public static MysqlPoolConfig buildConfig(SysDataSource sysDataSource) {
        return MysqlPoolConfig.builder()
                .id(sysDataSource.getId())
                .initialQuantity(sysDataSource.getInitialQuantity())
                .maximumFrequency(sysDataSource.getMaximumFrequency())
                .maximumQuantity(sysDataSource.getMaximumQuantity())
                .driverName(sysDataSource.getDriverName())
                .hostAddress(sysDataSource.getHostAddress())
                .hostPort(sysDataSource.getHostPort())
                .databaseName(sysDataSource.getDatabaseName())
                .dataConnectionParameter(sysDataSource.getDataConnectionParameter())
                .databaseUserName(sysDataSource.getDatabaseUserName())
                .databaseUserPassword(sysDataSource.getDatabaseUserPassword())
                .build();
    }


    /**
     * 获取数据库连接
     */
    public String getUrl() {
        StringBuilder urlSb = new StringBuilder(BaseConfig.MYSQLJDBCPRO);
        urlSb.append(this.hostAddress);
        urlSb.append(":");
        urlSb.append(this.hostPort);
        urlSb.append("/");
        urlSb.append(this.databaseName);
        urlSb.append("?");
        urlSb.append(this.dataConnectionParameter);
        return urlSb.toString();
    };



}
