package com.muyu.ruleengine.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.muyu.ruleengine.domain.Engineversion;
import com.muyu.ruleengine.domain.model.RuleContentModel;
import com.muyu.ruleengine.domain.req.*;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.ruleengine.domain.Ruleengine;
import com.muyu.ruleengine.service.RuleengineService;
import com.muyu.common.core.web.page.TableDataInfo;

/**
 * 规则引擎Controller
 *
 * @author WangLei
 * @date 2024-04-24
 */
@Api(tags = "规则引擎")
@RestController
@RequestMapping("/ruleengine")
public class RuleengineController extends BaseController {
    @Autowired
    private RuleengineService ruleengineService;


    /**
     * 编译
     * @param ruleContentReq 输入的内容
     */
    @PostMapping("/compiler")
    public Result compiler(@RequestBody RuleContentReq ruleContentReq) {
        return ruleengineService.compiler(ruleContentReq);
    }

    @GetMapping("/loader")
    public Result loader() {
        return ruleengineService.loader();
    }



    @GetMapping("/getRuleContent")
    public Result<RuleContentModel> getRuleContent(@RequestParam Long ruleId) {
        return ruleengineService.getRuleContent(ruleId);
    }


    /**
     * 查询规则引擎列表
     */
    @ApiOperation("获取规则引擎列表")
    @RequiresPermissions("ruleengine:ruleengine:list")
    @GetMapping("/list")
    public Result<TableDataInfo<Ruleengine>> list(RuleengineQueryReq ruleengineQueryReq) {
        startPage();
        List<Ruleengine> list = ruleengineService.list(Ruleengine.queryBuild(ruleengineQueryReq));
        return getDataTable(list);
    }

    /**
     * 导出规则引擎列表
     */
    @ApiOperation("导出规则引擎列表")
    @RequiresPermissions("ruleengine:ruleengine:export")
    @Log(title = "规则引擎", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, Ruleengine ruleengine) {
        List<Ruleengine> list = ruleengineService.list(ruleengine);
        ExcelUtil<Ruleengine> util = new ExcelUtil<Ruleengine>(Ruleengine.class);
        util.exportExcel(response, list, "规则引擎数据");
    }

    /**
     * 获取规则引擎详细信息
     */
    @ApiOperation("获取规则引擎详细信息")
    @RequiresPermissions("ruleengine:ruleengine:query")
    @GetMapping(value = "/{id}")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class)
    public Result<Ruleengine> getInfo(@PathVariable("id") Long id) {
        return Result.success(ruleengineService.getById(id));
    }

    /**
     * 新增规则引擎
     */
    @RequiresPermissions("ruleengine:ruleengine:add")
    @Log(title = "规则引擎", businessType = BusinessType.INSERT)
    @PostMapping
    @ApiOperation("新增规则引擎")
    public Result<String> add(@RequestBody RuleengineSaveReq ruleengineSaveReq) {
        return toAjax(ruleengineService.save(Ruleengine.saveBuild(ruleengineSaveReq)));
    }

    /**
     * 修改规则引擎
     */
    @RequiresPermissions("ruleengine:ruleengine:edit")
    @Log(title = "规则引擎", businessType = BusinessType.UPDATE)
    @PutMapping("/{id}")
    @ApiOperation("修改规则引擎")
    public Result<String> edit(@PathVariable Long id, @RequestBody RuleengineEditReq ruleengineEditReq) {
        return toAjax(ruleengineService.updateById(Ruleengine.editBuild(id,ruleengineEditReq)));
    }

    /**
     * 删除规则引擎
     */
    @RequiresPermissions("ruleengine:ruleengine:remove")
    @Log(title = "规则引擎", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    @ApiOperation("删除规则引擎")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = String.class, example = "1,2,3,4")
    public Result<String> remove(@PathVariable List<Long> ids) {
        return toAjax(ruleengineService.removeBatchByIds(ids));
    }
    @PostMapping("/findRuleEngineById")
    public Result findRuleEngineById(@RequestParam Long id) {
        return ruleengineService.findRuleEngineById(id);
    }

    @PostMapping("/addEngineversion")
    public Result addEngineversion(@RequestBody Engineversion engineversion) {
        return ruleengineService.addEngineversion(engineversion);
    }

    @PostMapping("/updateRuleActivate")
    public Result updateRuleActivate(@RequestBody Ruleengine ruleengine) {
        return ruleengineService.updateRuleActivate(ruleengine);
    }

    @PostMapping("/updateRuleStatus")
    public Result updateRuleStatus(@RequestBody Ruleengine ruleengine) {
        return ruleengineService.updateRuleStatus(ruleengine);
    }

    @PostMapping("/updateEngineCodeIng")
    public Result updateEngineCodeIng(@RequestBody Engineversion engineversion) {
        return ruleengineService.updateEngineCodeIng(engineversion);
    }
}
