package com.muyu.ruleengine.service.impl;

import java.lang.reflect.Method;
import java.net.URI;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.ruleengine.domain.Engineversion;
import com.muyu.ruleengine.domain.model.RuleContentModel;
import com.muyu.ruleengine.domain.model.RuleengineModel;
import com.muyu.ruleengine.domain.req.RuleContentReq;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.ruleengine.mapper.RuleengineMapper;
import com.muyu.ruleengine.domain.Ruleengine;
import com.muyu.ruleengine.service.RuleengineService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.web.bind.annotation.RequestBody;

import javax.tools.*;

/**
 * 规则引擎Service业务层处理
 *
 * @author WangLei
 * @date 2024-04-24
 */
@Slf4j
@Service
public class RuleengineServiceImpl extends ServiceImpl<RuleengineMapper, Ruleengine>  implements RuleengineService {


    @Autowired
    private RuleengineMapper ruleengineMapper;

    /**
     * 查询规则引擎列表
     *
     * @param ruleengine 规则引擎
     * @return 规则引擎
     */
    @Override
    public List<Ruleengine> list(Ruleengine ruleengine) {
        LambdaQueryWrapper<Ruleengine> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(ruleengine.getName())){
            queryWrapper.like(Ruleengine::getName, ruleengine.getName());
        }

        if (ObjUtils.notNull(ruleengine.getType())){
            queryWrapper.eq(Ruleengine::getType, ruleengine.getType());
        }

        if (ObjUtils.notNull(ruleengine.getIsActivate())){
            queryWrapper.eq(Ruleengine::getIsActivate, ruleengine.getIsActivate());
        }

        if (ObjUtils.notNull(ruleengine.getStatus())){
            queryWrapper.eq(Ruleengine::getStatus, ruleengine.getStatus());
        }
        if (ObjUtils.notNull(ruleengine.getRemark())){
            queryWrapper.eq(Ruleengine::getRemark, ruleengine.getRemark());
        }

        if (ObjUtils.notNull(ruleengine.getDescription())){
            queryWrapper.eq(Ruleengine::getDescription, ruleengine.getDescription());
        }

        if (ObjUtils.notNull(ruleengine.getCreateBy())){
            queryWrapper.eq(Ruleengine::getCreateBy, ruleengine.getCreateBy());
        }

        if (ObjUtils.notNull(ruleengine.getCreateTime())){
            queryWrapper.eq(Ruleengine::getCreateTime, ruleengine.getCreateTime());
        }

        if (ObjUtils.notNull(ruleengine.getUpdateBy())){
            queryWrapper.eq(Ruleengine::getUpdateBy, ruleengine.getUpdateBy());
        }

        if (ObjUtils.notNull(ruleengine.getUpdateTime())){
            queryWrapper.eq(Ruleengine::getUpdateTime, ruleengine.getUpdateTime());
        }
        return list(queryWrapper);
    }

    Long ruleId=0L;
    @Override
    public Result compiler(RuleContentReq ruleContentReq) {
        ruleId = ruleContentReq.getRuleId();
        String content = ruleContentReq.getRuleContent();
        // 指定输出目录
        String targetDirectory = "D:\\上海专高\\专高六\\test\\muyu-modules\\muyu-ruleengine\\muyu-ruleengine-server\\target\\classes\\com\\muyu\\ruleengine\\controller";

        JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
        try (StandardJavaFileManager fileManager = compiler.getStandardFileManager(null, null, null)) {
            // 创建一个内存中的源文件
            JavaFileObject sourceFileObject = new JavaSourceFromString("Test"+ruleId, content);

            // 编译选项
            Iterable<String> options = Arrays.asList("-d", targetDirectory);

            // 编译源代码
            JavaCompiler.CompilationTask task = compiler.getTask(null, fileManager, null, options, null, Arrays.asList(sourceFileObject));
            boolean success = task.call();

            if (success) {
                ruleengineMapper.addRulecontent(ruleContentReq);
                System.out.println("编译成功");
            } else {
                System.out.println("编译失败");
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.success("编译成功");
    }

    @Override
    public Result loader() {
        try {
            // 假设我们有一个名为com.example.MyClass的类文件
            // 注意：这里应该是.class文件的路径
            String classPath = "D:\\work\\2108A(3)\\cloud-server\\muyu-modules\\muyu-ruleengine\\muyu-ruleengine-server\\src\\main\\java\\com\\muyu\\ruleengine\\controller\\RuleengineController"+ruleId+".class";
            // 读取类文件
            byte[] classData = Files.readAllBytes(Paths.get(classPath));

            // 提取类名（不包括.class和路径）
            String className = classPath.substring(classPath.lastIndexOf('\\') + 1).replace(".class", "");

            // 使用自定义类加载器加载类
            MyClassLoader classLoader = new MyClassLoader();
            Class<?> clazz = classLoader.defineClassFromBytes(className, classData);

            // 使用反射创建实例并调用方法
            Object instance = clazz.getDeclaredConstructor().newInstance();
            Method execute = clazz.getMethod("execute", String.class);
            System.out.println(execute.invoke(instance, "zhangsan"));
            System.out.println(execute.invoke(instance, ""));
            // ... 调用方法等操作

        } catch (Exception e) {
            e.printStackTrace();
        }

        return Result.success("加载成功");
    }

    @Override
    public Result<RuleContentModel> getRuleContent(Long ruleId) {
        if (ObjUtils.notNull(ruleId)){
            RuleContentModel ruleContentModel=ruleengineMapper.getRuleContent(ruleId);
            return Result.success(ruleContentModel);
        }
        return null;
    }

    @Override
    public Result findRuleEngineById(Long id) {
        RuleengineModel ruleengine=ruleengineMapper.findRuleEngineById(id);
        List<Engineversion> engineversionList=ruleengineMapper.findEngineversionList(id);
        ruleengine.setEngineversionList(engineversionList);
        return Result.success(ruleengine);
    }

    @Override
    public Result addEngineversion(Engineversion engineversion) {
        ruleengineMapper.addEngineversion(engineversion);
        return Result.success("成功");
    }

    @Override
    public Result updateRuleActivate( Ruleengine ruleengine) {
        ruleengineMapper.updateRuleActivate(ruleengine);
        return Result.success("引擎激活状态修改成功");
    }

    @Override
    public Result updateRuleStatus(Ruleengine ruleengine) {
        ruleengineMapper.updateRuleStatus(ruleengine);
        return Result.success("引擎状态修改成功");
    }

    @Override
    public Result updateEngineCodeIng(Engineversion engineversion) {
        ruleengineMapper.updateEngineCodeIng(engineversion);
        return Result.success("编码保存成功");
    }


    public static class MyClassLoader extends ClassLoader {
        public Class<?> defineClassFromBytes(String name, byte[] data) {
            return defineClass(name, data, 0, data.length);
        }
    }
    private static Iterable<String> compilerOptions(String classOutputPath) {
        return Arrays.asList("-d", classOutputPath);
    }

    // 辅助类：表示内存中的源文件
    static class JavaSourceFromString extends SimpleJavaFileObject {
        final String code;

        JavaSourceFromString(String name, String code) {
            super(URI.create("string:///" + name.replace('.', '/') + Kind.SOURCE.extension), Kind.SOURCE);
            this.code = code;
        }

        @Override
        public CharSequence getCharContent(boolean ignoreEncodingErrors) {
            return code;
        }
    }
}
